using System;
using System.IO;
using Unity.Collections;
using UnityEngine;


namespace AssetGraph.Core.DataTypes
{
    [Serializable]

    public struct ImageData
    {
        public NativeArray<Color32> PixelData;
        public (int width, int height) Dimensions;
        public bool isRGBA;

        public ImageData(NativeArray<Color32> imageData, (int width, int height) widthHeight, bool isRgba)
        {
            PixelData = imageData;
            Dimensions = widthHeight;
            isRGBA = isRgba;
        }

        public ImageData(Texture2D sourceTexture)
        {
            PixelData = new NativeArray<Color32>(sourceTexture.GetPixelData<Color32>(0), Allocator.Persistent);
            Dimensions = (sourceTexture.width, sourceTexture.height);
            isRGBA = sourceTexture.format == TextureFormat.RGBA32;
        }

        public int Width => Dimensions.width;
        public int Height => Dimensions.height;

        public Texture2D ToTexture2D()
        {
            var texture = new Texture2D(Width, Height, TextureFormat.RGBA32, false);
            texture.LoadRawTextureData(PixelData);
            texture.Apply();
            return texture;
        }

        public void ExportPNG(string path)
        {
            Texture2D texture = this.ToTexture2D();
            byte[] data = texture.EncodeToPNG();

            if (data != null)
            {
                File.WriteAllBytes(path, data);
            }
        }
    }
}
