﻿using System;
using UnityEditor.Experimental.GraphView;
using UnityEngine;
using UnityEngine.UIElements;

namespace AssetGraph.Core.GraphElements
{
    [System.Serializable]
    public class AssetGraphStickyNoteData
    {
        [SerializeField] private string guid;
        [SerializeField] private string title;
        [SerializeField] private string content;
        [SerializeField] private Rect position;
        [SerializeField] private Vector2 size;


        public string GUID { get => guid; }

        public string Title { get => title; set => title = value; }

        public string Content { get => content; set => content = value; }
        public Rect Position { get => position; set => position = value; }
        public Vector2 Size { get => size; set => size = value; }

        public AssetGraphStickyNoteData(string title)
        {
            this.guid = Guid.NewGuid().ToString(); 
        }

    }
    
    
    public class AssetGraphStickyNote : StickyNote
    {
        private AssetGraphStickyNoteData data;
        public AssetGraphStickyNoteData Data => data;

        private AssetGraphViewWindow window;

        public AssetGraphStickyNote(AssetGraphStickyNoteData data, AssetGraphViewWindow window)
        {
            this.theme = StickyNoteTheme.Black;
            this.window = window;
            this.data = data;

            this.title = data.Title;
            this.contents = data.Content;
            this.SetPosition(data.Position);
            
            var le = this.Q<TextField>("contents-field");

            le.RegisterValueChangedCallback(evt =>
            {
                this.data.Content = evt.newValue;
            });
        }

        public override void OnResized()
        {
            base.OnResized();
            data.Size = new Vector2(this.resolvedStyle.width, this.resolvedStyle.height);
        }

        public void SavePosition() => data.Position = GetPosition();
    }

}