using System;
using System.Linq;
using System.Reflection;
using AssetGraph.Nodes;
using UnityEditor.Experimental.GraphView;
using UnityEngine;
using UnityEngine.UIElements;

namespace AssetGraph.Core
{
    public class AssetGraphPort : Port
    {
        public FieldInfo fieldInfo;
        private VisualElement _exposedPropertyContainer;

        public delegate void OnPortConnectedEvent();
        public OnPortConnectedEvent OnPortConnected;
        
        public delegate void OnPortDisconnectedEvent();
        public OnPortDisconnectedEvent OnPortDisconnected;
        
        public VisualElement ExposedPropertyContainer
        {
            set
            {
                _exposedPropertyContainer = value;
            }
            get
            {
                return _exposedPropertyContainer;
            }
        }
        
        protected AssetGraphPort(Orientation portOrientation, Direction portDirection, Capacity portCapacity, Type type) : base(portOrientation, portDirection, portCapacity, type)
        {
            this.tooltip = $"{this.portType.ToString()}";
        }

        public override void Connect(Edge edge)
        {
            if (this.node is AssetGraphNodeEditor IPTVis && IPTVis.GraphNode is GenericConnection)
            {
                foreach (var port in IPTVis.AllPorts)
                {
                    port.portType = edge.output.portType;
                }
            }
            
            base.Connect(edge);
            OnPortConnected?.Invoke();
        }

        public override void Disconnect(Edge edge)
        {
            AssetGraphNodeEditor IPTVis = this.node as AssetGraphNodeEditor;

            if (IPTVis != null && IPTVis.GraphNode is GenericConnection bleh)
            {
                bool allPortsDisconnected = true;

                foreach (var port in IPTVis.AllPorts)
                {
                    if (port.connections != null && port.connections.Any())
                    {
                        allPortsDisconnected = false;
                        break;
                    }
                }

                if (allPortsDisconnected)
                {
                    foreach (var port in IPTVis.AllPorts)
                    {
                        port.portType = typeof(object);
                    }

                    bleh.InternalType = null;
                }
            }

            
            base.Disconnect(edge);
            OnPortDisconnected?.Invoke();
        }


        private void PublicOnConnected(Port obj)
        {
            throw new NotImplementedException();
        }

        public static AssetGraphPort Create(IEdgeConnectorListener connectorListener, bool isInput, Type type)
        {
            var port = new AssetGraphPort(Orientation.Horizontal, isInput ? Direction.Input : Direction.Output,
                isInput ? Capacity.Single : Capacity.Multi, type)
            {
                m_EdgeConnector = new EdgeConnector<Edge>(connectorListener),
            };
            port.AddManipulator(port.m_EdgeConnector);
            return port;
        }

    }
}