﻿using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace AssetGraph.Nodes
{
    [NodeInfo("Invert Channel", "Adjustments/InvertChannel", true, docuementationURL: "https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FInvertChannel")]
    public class ChannelInvertNode : AssetGraphNode
    {
        [Core.Attributes.Input("Input Channel")]
        public SplitChannelData inputChannel;

        [Core.Attributes.Output("Output Channel")]
        public SplitChannelData outputChannel;

        public override void Process()
        {
            NativeArray<byte> input = new NativeArray<byte>(inputChannel.ChannelData, Allocator.Persistent);
            NativeArray<byte> output = new NativeArray<byte>(inputChannel.ChannelData.Length, Allocator.Persistent);

            InvertChannelJob job = new InvertChannelJob
            {
                input = input,
                output = output
            };

            job.Run();

            byte[] outputArray = output.ToArray();

            outputChannel = new SplitChannelData(outputArray, (inputChannel.Width, inputChannel.Height));
        }
    }

    [BurstCompile]
    public struct InvertChannelJob : IJob
    {
        [ReadOnly] public NativeArray<byte> input;
        [WriteOnly] public NativeArray<byte> output;

        public void Execute()
        {
            int length = input.Length;
            for (int i = 0; i < length; i++)
            {
                output[i] = (byte)(255 - input[i]);
            }
        }
    }
}