using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;



namespace AssetGraph.Nodes
{
    [NodeInfo("Desaturate", "Adjustments/Desaturate", true, docuementationURL: "https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FDesaturate")]
    public class Texture2DDesaturate : AssetGraphNode
    {
        [Core.Attributes.Input("")]
        public ImageData inputTexture;

        [Core.Attributes.Output("")] 
        public ImageData outputTexture;

        public override void Process()
        {
            NativeArray<Color32> output = new NativeArray<Color32>(inputTexture.PixelData.Length, Allocator.Persistent);

            DesaturateJob job = new DesaturateJob
            {
                pixels = inputTexture.PixelData,
                outputPixels = output
            };

            job.Run();

            outputTexture = new ImageData(output, (inputTexture.Width, inputTexture.Height), inputTexture.isRGBA);
        }
    }

    public struct DesaturateJob : IJob
    {
        [ReadOnly] public NativeArray<Color32> pixels;  
        public NativeArray<Color32> outputPixels;  

        public void Execute()
        {
            for (int i = 0; i < pixels.Length; i++)
            {
                Color32 pixel = pixels[i];

                byte grayValue = (byte)(0.2989f * pixel.r + 0.5870f * pixel.g + 0.1140f * pixel.b);

                outputPixels[i] = new Color32(grayValue, grayValue, grayValue, pixel.a);
            }
        }
    }
}