﻿using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace AssetGraph.Nodes
{
    [NodeInfo("Invert", "Adjustments/Invert", true, docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FInvert")]
    public class Texture2DInvert : AssetGraphNode
    {
        [Core.Attributes.Input("")]
        public ImageData inputTexture;
        
        [Core.Attributes.Output("")]
        public ImageData outputTexture;

        public override void Process()
        {
            NativeArray<Color32> output = new NativeArray<Color32>(inputTexture.PixelData.Length, Allocator.Persistent);

            InvertJob job = new InvertJob
            {
                pixels = inputTexture.PixelData,
                outputPixels = output
            };

            job.Run();

            outputTexture = new ImageData(output, (inputTexture.Width, inputTexture.Height), inputTexture.isRGBA);
        }
    }

    [BurstCompile]
    public struct InvertJob : IJob
    {
        [ReadOnly] public NativeArray<Color32> pixels;
        [WriteOnly] public NativeArray<Color32> outputPixels;
        
        public void Execute()
        {
            int length = pixels.Length;

            for (int i = 0; i < length; i++)
            {
                Color32 pixel = pixels[i];
                outputPixels[i] = new Color32(
                    (byte)(255 - pixel.r),
                    (byte)(255 - pixel.g),
                    (byte)(255 - pixel.b),
                    pixel.a);
            }
        }
    }
}