﻿using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace AssetGraph.Nodes
{
    [NodeInfo("RGBA Split", "Channels/RGBA Split", true, docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FRGBASplit")]
    public class RGBASplit : AssetGraphNode
    {
        [Core.Attributes.Input("")] 
        public ImageData inputTexture;

        [Core.Attributes.Output("R")] 
        public SplitChannelData r;
        [Core.Attributes.Output("G")] 
        public SplitChannelData g;
        [Core.Attributes.Output("B")] 
        public SplitChannelData b;
        [Core.Attributes.Output("A")] 
        public SplitChannelData a;

        public override void Process()
        {
            int length = inputTexture.PixelData.Length;;

            // Allocate NativeArrays for the pixel data and the individual RGBA channels
            NativeArray<Color32> pixelData = inputTexture.PixelData;
            NativeArray<byte> rChannel = new NativeArray<byte>(length, Allocator.Persistent);
            NativeArray<byte> gChannel = new NativeArray<byte>(length, Allocator.Persistent);
            NativeArray<byte> bChannel = new NativeArray<byte>(length, Allocator.Persistent);
            NativeArray<byte> aChannel = new NativeArray<byte>(length, Allocator.Persistent);

            // Create the job and set up the input/output data
            RGBAJob rgbaJob = new RGBAJob
            {
                pixelData = pixelData,
                rData = rChannel,
                gData = gChannel,
                bData = bChannel,
                aData = aChannel,
                length = length
            };

            rgbaJob.Run();

            // Store the results in the SplitChannelData objects
            r = new SplitChannelData(rChannel.ToArray(), (inputTexture.Width, inputTexture.Height));
            g = new SplitChannelData(gChannel.ToArray(), (inputTexture.Width, inputTexture.Height));
            b = new SplitChannelData(bChannel.ToArray(), (inputTexture.Width, inputTexture.Height));
            a = new SplitChannelData(aChannel.ToArray(), (inputTexture.Width, inputTexture.Height));
        }
    }
    
    // Job struct for processing the image data
    [BurstCompile]
    struct RGBAJob : IJob
    {
        public NativeArray<Color32> pixelData;
        public NativeArray<byte> rData;
        public NativeArray<byte> gData;
        public NativeArray<byte> bData;
        public NativeArray<byte> aData;
        public int length;

        // Execute method that splits the RGBA channels
        public void Execute()
        {
            for (int i = 0; i < length; i++)
            {
                Color32 pixel = pixelData[i];
                rData[i] = pixel.r;
                gData[i] = pixel.g;
                bData[i] = pixel.b;
                aData[i] = pixel.a;
            }
        }
    }
}