﻿using System.ComponentModel.Composition.Primitives;
using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Collections;
using UnityEditor;
using UnityEngine;

namespace AssetGraph.Nodes
{
    [NodeInfo("Texture Export", "Export/Texture Export", true,docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FTexture2DOutput")]
    public class Texture2DOutput : AssetGraphNode
    {
        [Core.Attributes.Input("")] public ImageData inputPixels;
        [Core.Attributes.Input("File Name")] public string fileName;
        [Core.Attributes.Input("File Directory")] public string fileDirectory;

        public enum ExportType
        {
            Texture2D,
            PNG
        }
        
        [Core.Attributes.Input("Export Type")] public ExportType exportType;
        [Core.Attributes.Input("Texture Type")] public TextureImporterType textureType;
        
        [Core.Attributes.Output("Output Texture")] public Texture2D textureOutput;

        public override void Process()
        {
            
            switch (exportType)
            {
                case ExportType.Texture2D:
                    string pathT2D = $"{fileDirectory}/{fileName}.asset";
                    AssetDatabase.CreateAsset(inputPixels.ToTexture2D(), pathT2D);
                    textureOutput = AssetDatabase.LoadAssetAtPath<Texture2D>(pathT2D);
                    break;
                case ExportType.PNG:
                    string pathPNG = $"{fileDirectory}/{fileName}.png";
                    inputPixels.ExportPNG(pathPNG);
                    
                    AssetDatabase.ImportAsset(pathPNG);
                    
                    TextureImporter textureImporter = AssetImporter.GetAtPath(pathPNG) as TextureImporter;
                    if (textureImporter != null)
                    {
                        textureImporter.textureType = textureType;
                        EditorUtility.SetDirty(AssetDatabase.LoadAssetAtPath<Texture2D>(pathPNG));
                        textureImporter.SaveAndReimport();
                    }
                    AssetDatabase.ImportAsset(pathPNG);
                    
                    textureOutput = AssetDatabase.LoadAssetAtPath<Texture2D>(pathPNG);
                    break;
            }
        }
    }
}