﻿using System.IO;
using System.Security.Policy;
using AssetGraph.Core.Attributes;
using UnityEditor;
using UnityEngine;

namespace AssetGraph.Nodes
{
    public class FromPathNode<T> : AssetGraphNode where T : UnityEngine.Object
    {
        [Core.Attributes.Input("Path")] public string path;
        [Core.Attributes.Output("Output")] public T output;

        public override void Process()
        {
            if (string.IsNullOrEmpty(path))
            {
                throw new System.ArgumentException("Path is empty or null. Cannot load asset.");
            }

            output = AssetDatabase.LoadAssetAtPath<T>(path);

            if (output == null)
            {
                throw new System.Exception($"Failed to load asset of type {typeof(T)} at path: {path}.");
            }
        }

        public override void CleanUp()
        {
            output = null;
        }
    }
    
    [NodeInfo("Load Texture2D From path", "Load From Path/Texture2D", docuementationURL: "https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImages%2FFromPathNode")]
    public class Texture2DFromPath : FromPathNode<Texture2D> { }
    
    [NodeInfo("Load Mesh From path", "Load From Path/Mesh", docuementationURL: "https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImages%2FFromPathNode")]
    public class MeshFromPath : FromPathNode<Mesh> { }
    
    [NodeInfo("Load Prefab From path", "Load From Path/Prefab", docuementationURL: "https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImages%2FFromPathNode")]
    public class PrefabFromPath : FromPathNode<GameObject> { }
}