﻿using System.IO;
using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using Unity.Collections;
using UnityEditor;
using UnityEngine;

namespace AssetGraph.Nodes
{
    [NodeInfo("Texture Import", "Imports/Import Texture", docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FTexture2DImport")]
    public class Texture2DImport : AssetGraphNode
    {
        [Core.Attributes.Input("")]
        public Texture2D textureImport;
        
        [Core.Attributes.Output("")]
        public ImageData textureOutput;
        [Core.Attributes.Output("File Name")]
        public string fileName;
        [Core.Attributes.Output("File Directory")]
        public string filePath;

        public override void Process()
        {
            if (this.textureImport != null)
            {
                TextureImporter textureImporter = AssetImporter.GetAtPath(AssetDatabase.GetAssetPath(textureImport)) as TextureImporter;
            
                TextureImporterPlatformSettings texset = textureImporter.GetDefaultPlatformTextureSettings();
                texset.format=TextureImporterFormat.RGBA32;
                texset.maxTextureSize=16384;
                textureImporter.SetPlatformTextureSettings(texset);
            
                TextureImporterSettings settings = new TextureImporterSettings();
                textureImporter.ReadTextureSettings(settings);
                settings.readable = true;
                textureImporter.SetTextureSettings(settings);
                EditorUtility.SetDirty(textureImport);
                textureImporter.SaveAndReimport();
                
                
                this.textureOutput = new ImageData(textureImport);
                this.fileName = textureImport.name;
                this.filePath = Path.GetDirectoryName(AssetDatabase.GetAssetPath(textureImport));
            }
            // else
            //     Debug.LogError("UH!");
        }
    }
}