﻿using AssetGraph.Core.Attributes;
using AssetGraph.Core.DataTypes;
using AssetGraph.Core.GraphElements;
using Unity.Burst;
using Unity.Collections;
using Unity.Jobs;
using UnityEngine;

namespace AssetGraph.Nodes
{
    public class SingleChannelColor
    {
        [NodeInfo("Channel Color", "Utility/Channel Color", false, docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImages%2FSingleChannelColor")]
        public class SingleColorChannel : AssetGraphNode
        {
            [Core.Attributes.Input("Color")] public GreyscaleValue range;

            [Core.Attributes.Input("Width")] public int Width;

            [Core.Attributes.Input("Height")] public int Height;

            [Core.Attributes.Output("Color")] public SplitChannelData OutputColor;

            public override void Process()
            {
                int pixelCount = Width * Height;
    
                NativeArray<byte> outputData = new NativeArray<byte>(pixelCount, Allocator.Persistent);

                CreateSingleColorJob job = new CreateSingleColorJob
                {
                    color32 = range.value,
                    outputData = outputData,
                    width = Width,
                    height = Height
                };

                job.Run();

                OutputColor = new SplitChannelData(outputData.ToArray(), (Width, Height));
            }

            [BurstCompile]
            struct CreateSingleColorJob : IJob
            {
                public int color32;
                [WriteOnly]
                public NativeArray<byte> outputData;
                public int width;
                public int height;

                public void Execute()
                {
                    // More efficient linear write pattern
                    for (int i = 0; i < width*height; i++)
                    {
                        outputData[i] = (byte)color32;
                    }
                }
            }
        }
    }
}
