﻿using AssetGraph.Nodes;
using UnityEngine;
using AssetGraph.Core;
using AssetGraph.Core.Attributes;

namespace AssetGraph.Nodes
{
    public class VariableNodeBase : AssetGraphNode
    {
        [Core.Attributes.Input("Variable Name")] public string variableName;
    }
    
    public class VariableNode<T> : VariableNodeBase
    {
        public T input;
        [Core.Attributes.Output("Output")] public T output;

        public override void Process()
        {
            if(input != null)
                this.output = input;
            else
                throw new System.NullReferenceException($"Input to VariableNode<{typeof(T).Name}> is null");
        }

        public override void CleanUp()
        {
            input = default;
            output = default;
        }
    }

    [NodeInfo("Texture2D Variable", "Variable/Texture2D", docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FVariableNode")]
    public class Texture2DVariableNode : VariableNode<Texture2D> { }
    
    [NodeInfo("Mesh Variable", "Variable/Mesh", docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FVariableNode")]
    public class MeshVariableNode : VariableNode<Mesh> { }
    
    [NodeInfo("Prefab Variable", "Variable/Prefab", docuementationURL:"https://git.sam-green.dev/Chromum/UnityImageProcessing_Package/wiki/Nodes%2FImage%2FVariableNode")]
    public class PrefabVariableNode : VariableNode<GameObject> { }
}