using System.IO;
using AssetGraph.Core;
using UnityEngine;
using UnityEditor;
using AssetGraph.Core.Attributes;
using AssetGraph.Nodes;
using UnityEngine.UIElements;

namespace ImageProcessingGraph.Editor.Nodes.MazStuff
{
    [NodeInfo("Debug Log", "Debug/Debug Log", editorType: typeof(DebugLogNodeEditor))]
    public class DebugLogNode : AssetGraphNode
    {
        [AssetGraph.Core.Attributes.Input("Log Text")]
        public string logText;

        public delegate void OnNodeUpdated();
        public OnNodeUpdated onNodeUpdated;
        
        public override void Process()
        { 
            Debug.Log(logText);
            onNodeUpdated?.Invoke();
        }
    }

    public class DebugLogNodeEditor : AssetGraphNodeEditor
    {
        DebugLogNode logNode;
        Label label;
        
        public DebugLogNodeEditor(AssetGraphNode node, AssetGraphViewWindow window) : base(node, window)
        {
            logNode = node as DebugLogNode;
            logNode.onNodeUpdated += UpdateEditor;
            label = new Label();
            label.style.flexGrow = 1;
            label.style.flexShrink = 1;
            label.style.paddingBottom = label.style.paddingTop = label.style.paddingLeft = label.style.paddingRight = 6;
            label.style.backgroundColor = new Color(0.24705f, 0.24705f, 0.24705f, 0.8039216f);
            label.style.whiteSpace = WhiteSpace.Normal;

            VisualElement bar = new VisualElement();
            bar.name = "divider";
            bar.AddToClassList("horizontal");

            var contents = this.Q("contents");

            contents.Add(bar);
            contents.Add(label);
        }

        public void UpdateEditor()
        {
            label.text = logNode.logText;
        }
    }
}