using System.IO;
using UnityEngine;
using UnityEditor;
using AssetGraph.Core.Attributes;
using AssetGraph.Nodes;

namespace ImageProcessingGraph.Editor.Nodes.MazStuff
{
    [NodeInfo("Create Prefab", "Prefab/Create")]
    public class CreatePrefabNode : AssetGraphNode
    {
        [AssetGraph.Core.Attributes.Input("Prefab Name")]
        public string prefabName;
        
        [AssetGraph.Core.Attributes.Input("Directory")]
        public string directory;
        
        [AssetGraph.Core.Attributes.Input("")]
        public Mesh mesh;
        
        [AssetGraph.Core.Attributes.Input("Submesh Count")]
        public int submeshCount = 1;

        [AssetGraph.Core.Attributes.Input("")]
        public Material material;

        [AssetGraph.Core.Attributes.Output("Prefab")]
        public GameObject prefab;

        public override void Process()
        {
            GameObject newGO = new GameObject(prefabName);
            var prefabPath = Path.Combine(directory, $"{prefabName}.prefab");
            PrefabUtility.SaveAsPrefabAsset(newGO, prefabPath);
            Object.DestroyImmediate(newGO);

            using (var editScope = new PrefabUtility.EditPrefabContentsScope(prefabPath))
            {
                var root = editScope.prefabContentsRoot;
                var meshRenderer = root.AddComponent<MeshRenderer>();
                var meshFilter = root.AddComponent<MeshFilter>();
                meshFilter.sharedMesh = mesh;
                if (submeshCount > 1)
                {
                    Material[] materials = new Material[submeshCount];
                    for (int i = 0; i < submeshCount; i++)
                    {
                        materials[i] = material;
                    }

                    meshRenderer.sharedMaterials = materials;
                }
                else
                {
                    meshRenderer.sharedMaterial = material;
                }
            }

            prefab = AssetDatabase.LoadAssetAtPath<GameObject>(prefabPath);
        }
    }
}