using System.IO;
using AssetGraph.Core.Attributes;
using AssetGraph.Nodes;
using ImageProcessingGraph.Editor;
using UnityEditor;
using UnityEngine;

namespace ImageProcessingGraph.Editor.Nodes.MazStuff
{
    [NodeInfo("Model Import", "Imports/Import Model", true)]
    public class ModelImportNode : AssetGraphNode
    {
        [AssetGraph.Core.Attributes.Input("Model")]
        public GameObject modelImport;
        
        [AssetGraph.Core.Attributes.Output("")]
        public Mesh mesh;
        
        [AssetGraph.Core.Attributes.Output("")]
        public Material material;
       
        [AssetGraph.Core.Attributes.Output("Model Name")]
        public string modelName;
        
        [AssetGraph.Core.Attributes.Output("Directory")]
        public string directory;
        
        public override void Process()
        {
            var modelPath = AssetDatabase.GetAssetPath(modelImport);
            var allAssets = AssetDatabase.LoadAllAssetsAtPath(modelPath);

            material = null;
            mesh = null;
            foreach (var asset in allAssets)
            {
                if (material && mesh) break;
                
                if (!material && asset is Material foundMaterial)
                {
                    material = foundMaterial;
                }
                if (!mesh && asset is Mesh foundMesh)
                {
                    mesh = foundMesh;
                }
            }

            modelName = modelImport.name;
            directory = Path.GetDirectoryName(modelPath);
        }
    }
    
    
    [NodeInfo("Mesh Info", "Mesh/Mesh Info", true)]
    public class MeshInfo : AssetGraphNode
    {
        [AssetGraph.Core.Attributes.Input("")]
        public Mesh mesh;
        
        [AssetGraph.Core.Attributes.Output("Submeshes")]
        public int submeshes;
        
        public override void Process()
        {
            submeshes = mesh.subMeshCount;
        }
    }
}