using System.Collections.Generic;
using ImageProcessingGraph.Editor;
using UnityEditor.Experimental.GraphView;
using UnityEditor.MemoryProfiler;
using UnityEngine;
using UnityEngine.UIElements;

namespace AssetGraph.Core
{

    public class AssetGraphEdgeConnector : IEdgeConnectorListener
    {
        private GraphViewChange m_GraphViewChange;
        private List<Edge> m_EdgesToCreate;
        private List<GraphElement> m_EdgesToDelete;

        private AssetGraphViewWindow window;

        public AssetGraphEdgeConnector(AssetGraphViewWindow window)
        {
            this.m_EdgesToCreate = new List<Edge>();
            this.m_EdgesToDelete = new List<GraphElement>();
            this.m_GraphViewChange.edgesToCreate = this.m_EdgesToCreate;
            this.window = window;
        }

        public void OnDropOutsidePort(Edge edge, Vector2 position)
        {
            window.searchProvider.target = (VisualElement)window.focusController.focusedElement;
            
            var mousePos =
                window.ChangeCoordinatesTo(window, window.cachedMousePos - window.Window.position.position);
            var graphMousePosition = window.contentViewContainer.WorldToLocal(mousePos);

            var searchProviderInstance = ScriptableObject.CreateInstance<AssetGraphSearchProvider.CustomSearchProviderForEdge>();
            searchProviderInstance.Init(window.searchProvider, edge, (AssetGraphPort)edge.input, (AssetGraphPort)edge.output);

            SearchWindow.Open(
                new SearchWindowContext(GUIUtility.GUIToScreenPoint(Event.current.mousePosition)),
                searchProviderInstance
            );
    
            // Remove connections as you did
            List<GraphConnection> connections = new List<GraphConnection>();
            foreach (var conn in window.asset.Connections)
            {
                if (conn.internalEdge == edge)
                    connections.Add(conn);
            }
            foreach (var VARIABLE in connections)
            {
                window.asset.Connections.Remove(VARIABLE);
            }

            if (edge.input != null)
                if (edge.input.node != null)
                    ((AssetGraphNodeEditor)edge.input.node).ToggleExposedVariable((AssetGraphPort)edge.input, true);
        }



        public void OnDrop(UnityEditor.Experimental.GraphView.GraphView graphView, Edge edge)
        {
            List<GraphConnection> connections = new List<GraphConnection>();
            
            foreach (var conn in window.asset.Connections)
            {
                if (conn.internalEdge == edge)
                {
                    connections.Add(conn);
                }
            }
            
            foreach (var VARIABLE in connections)
            {
                window.asset.Connections.Remove(VARIABLE);
            }
            
            ((AssetGraphNodeEditor)edge.input.node).ToggleExposedVariable((AssetGraphPort)edge.input, true);
            
            
            this.m_EdgesToCreate.Clear();
            this.m_EdgesToCreate.Add(edge);
            this.m_EdgesToDelete.Clear();
            if (edge.input.capacity == Port.Capacity.Single)
            {
                foreach (Edge connection in edge.input.connections)
                {
                    if (connection != edge)
                        this.m_EdgesToDelete.Add((GraphElement)connection);
                }
            }

            if (edge.output.capacity == Port.Capacity.Single)
            {
                foreach (Edge connection in edge.output.connections)
                {
                    if (connection != edge)
                        this.m_EdgesToDelete.Add((GraphElement)connection);
                }
            }

            if (this.m_EdgesToDelete.Count > 0)
                graphView.DeleteElements((IEnumerable<GraphElement>)this.m_EdgesToDelete);
            List<Edge> edgesToCreate = this.m_EdgesToCreate;
            if (graphView.graphViewChanged != null)
                edgesToCreate = graphView.graphViewChanged(this.m_GraphViewChange).edgesToCreate;
            foreach (Edge edge1 in edgesToCreate)
            {
                graphView.AddElement((GraphElement)edge1);
                edge.input.Connect(edge1);
                edge.output.Connect(edge1);
            }
        }
    }
}